package com.jkcredit.invoice.service.upService.impl;

import cn.com.taiji.common.manager.net.http.binclient.ApiRequestException;
import cn.com.taiji.sdk.comm.ETCCommHelper;
import cn.com.taiji.sdk.model.comm.protocol.eoms.company.*;
import com.jkcredit.invoice.service.upService.CommInterFace;
import com.jkcredit.invoice.util.RespR;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.io.IOException;
@Service("commInterFace")
@Slf4j
public class CommInterFaceImpl implements CommInterFace{

    
    @Override
    public RespR<CompanyQueryResponse> companyQuery(CompanyQueryRequest companyQueryRequest) {
        String fileName = companyQueryRequest.getFilename();
        long startTime = System.currentTimeMillis();
        try {
            log.info("CommInterFaceImpl.companyQuery上游接口请求参数：{}",  companyQueryRequest.toString());
            //(4)指定协议的响应模型（IssuerUploadResponse），调用upload 发送数据
            CompanyQueryResponse response= ETCCommHelper.upload(fileName, companyQueryRequest, CompanyQueryResponse.class);
            log.info("CommInterFaceImpl.companyQuery上游接口返回：{},请求参数：{},cost={}ms", response.toString(), companyQueryRequest.toString(),System.currentTimeMillis()-startTime);
            return new RespR<>(response);
        } catch (IOException e) {
            log.error("CommInterFaceImpl.companyQuery:{}",e.getMessage()+ " ,请求参数："+companyQueryRequest.toString()+" ,cost="+(System.currentTimeMillis()-startTime)+"ms");
            e.printStackTrace();
            return new RespR(false,"网络异常,请联系管理人员");
        }catch (ApiRequestException apie) {
            log.error("CommInterFaceImpl.companyQuery:{}",apie.getMessage()+ " ,请求参数："+companyQueryRequest.toString()+" ,cost="+(System.currentTimeMillis()-startTime)+"ms");
            apie.printStackTrace();
            return new RespR(false,apie.getMessage());
        }catch (Exception e) {
            log.error("CommInterFaceImpl.companyQuery:{}",e.getMessage()+ " ,请求参数："+companyQueryRequest.toString()+" ,cost="+(System.currentTimeMillis()-startTime)+"ms");
            e.printStackTrace();
            return new RespR(false,"网络异常,请联系管理人员");
        }
    }

    @Override
    public RespR<CompanyAddResponse> addCompany(CompanyAddRequest companyAddRequest) {
        String fileName = companyAddRequest.getFilename();
        long startTime = System.currentTimeMillis();

        try {
            log.info("CommInterFaceImpl.addCompany请求参数：{}", companyAddRequest.toString());
            //(4)指定协议的响应模型（IssuerUploadResponse），调用upload 发送数据
            CompanyAddResponse response=ETCCommHelper.upload(fileName, companyAddRequest, CompanyAddResponse.class);
            log.info("CommInterFaceImpl.addCompany上游接口返回：{},请求参数：{},cost={}ms", response.toString(), companyAddRequest.toString(),System.currentTimeMillis()-startTime);
            return new RespR<>(response);
        } catch (IOException e) {
            log.error("CommInterFaceImpl.addCompany：{}", e.getMessage()+ " ,请求参数："+companyAddRequest.toString()+" ,cost="+(System.currentTimeMillis()-startTime)+"ms");
            e.printStackTrace();
            return new RespR(false,"网络异常,请联系管理人员");
        }catch (ApiRequestException apie) {
            log.error("CommInterFaceImpl.addCompany：{}", apie.getMessage()+ " ,请求参数："+companyAddRequest.toString()+" ,cost="+(System.currentTimeMillis()-startTime)+"ms");
            apie.printStackTrace();
            return new RespR(false,apie.getMessage());
        }
    }

    @Override
    public RespR<B2bContractAddResponse> b2bContractAdd(B2bContractAddRequest contractAddRequest) {
        String fileName = contractAddRequest.getFilename();
        long startTime = System.currentTimeMillis();

        try {
            log.info("CommInterFaceImpl.b2bContractAdd上游请求参数：{}",contractAddRequest.toString());
            //(4)指定协议的响应模型（IssuerUploadResponse），调用upload 发送数据
            B2bContractAddResponse response=ETCCommHelper.upload(fileName, contractAddRequest, B2bContractAddResponse.class);
            log.info("CommInterFaceImpl.b2bContractAdd上游接口返回：{},请求参数：{},cost={}ms", response.toString(), contractAddRequest.toString(),System.currentTimeMillis()-startTime);
            log.info("conName"+response.getContrantNum());
            return new RespR<>(response);
        } catch (IOException e) {
            e.printStackTrace();
            log.error("CommInterFaceImpl.b2bContractAdd：{}",e.getMessage()+ " ,请求参数："+contractAddRequest.toString()+" ,cost="+(System.currentTimeMillis()-startTime)+"ms");
            return new RespR(false,"网络异常,请联系管理人员");
        }catch (ApiRequestException apie) {
            log.error("CommInterFaceImpl.b2bContractAdd：{}",apie.getMessage()+ " ,请求参数："+contractAddRequest.toString()+" ,cost="+(System.currentTimeMillis()-startTime)+"ms");
            apie.printStackTrace();
            return new RespR(false,apie.getMessage());
        }
    }

    @Override
    public RespR<B2bContractQueryResponse> b2bContractQuery(B2bContractQueryRequest b2bContractQueryRequest) {
        String fileName = b2bContractQueryRequest.getFilename();
        long startTime = System.currentTimeMillis();
        try {
            log.info("CommInterFaceImpl.b2bContractQuery上游请求参数：{}",  b2bContractQueryRequest.toString());
            //(4)指定协议的响应模型（IssuerUploadResponse），调用upload 发送数据
            B2bContractQueryResponse response=ETCCommHelper.upload(fileName, b2bContractQueryRequest, B2bContractQueryResponse.class);
            log.info("CommInterFaceImpl.b2bContractQuery上游接口返回：{},请求参数：{},cost={}ms", response.toString(), b2bContractQueryRequest.toString(),System.currentTimeMillis()-startTime);

            return new RespR<>(response);
        } catch (IOException e) {
            e.printStackTrace();
            log.error("CommInterFaceImpl.b2bContractQuery：{}",  e.getMessage()+ " ,请求参数："+b2bContractQueryRequest.toString()+" ,cost="+(System.currentTimeMillis()-startTime)+"ms");
            return new RespR(false,"网络异常,请联系管理人员");
        }catch (ApiRequestException apie) {
            apie.printStackTrace();
            log.error("CommInterFaceImpl.b2bContractQuery：{}",  apie.getMessage()+ " ,请求参数："+b2bContractQueryRequest.toString()+" ,cost="+(System.currentTimeMillis()-startTime)+"ms");
            return new RespR(false,apie.getMessage());
        }
    }

}
